// -*- C++ -*-
/*!
 * @file SdoServiceAdmin.h
 * @brief SDO service administration class
 * @date $Date$
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2011
 *     Noriaki Ando
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: SdoConfiguration.cpp 1971 2010-06-03 08:46:40Z n-ando $
 *
 */

#ifndef RTC_SDOSERVICEADMIN_H
#define RTC_SDOSERVICEADMIN_H

#include <coil/Mutex.h>
#include <coil/Factory.h>

#include <rtm/idl/SDOPackageStub.h>
#include <rtm/SystemLogger.h>

namespace RTC
{
  class RTObject_impl;
  class SdoServiceConsumerBase;

  /*!
   * @if jp
   *
   * @class SDO service administration class
   * @brief SDO service 饹
   *
   * Υ饹ϡSDO Service 뤿Υ饹Ǥ롣SDO
   * Service  OMG SDO Specification ˤƤ롢SDO
   * εǽΤ󶡤ޤ׵᤹륵ӥΰĤǤ롣ܺ٤ϻͤˤ
   * Ƥʤܥ饹ǤϰʲΤ褦˿񤦥ӥ
   * ΤȤ뤿Υ饹ܥ饹Ǥ롣
   *
   * SDO Service ˤƤϡSDO/RTC˽ͭ졢Υӥ
   * Τ SDO Service Provider¾SDO/RTC䥢ץꥱ
   * 륵ӥ֥ȤλȤꡢεǽѤ
   * ΤSDO Service Consumer ȸƤ֡
   *
   * SDO Service Provider ¾Υץꥱ󤫤ƤФ졢SDO/RTC
   * εǽ˥뤿Ѥ롣¾SDO/RTCޤϥץꥱ
   * ϡ
   *
   * - SDO::get_service_profiles ()
   * - SDO::get_service_profile (in UniqueIdentifier id)
   * - SDO::get_sdo_service (in UniqueIdentifier id) 
   *
   * Τ줫Υڥ졼ˤꡢServiceProfile ޤ SDO
   * Service λȤǽѤ뤿Υڥ졼Ƥӽ
   * ¾SDO/RTCޤϥץꥱǤλȤ˴ǤդΥ
   * ߥ󥰤ǹԤ졢ӥ¦ǤϡɤɤȤƤ
   * Τ뤳ȤϤǤʤǡSDO/RTC¦⡢ǤդΥߥ󥰤ǥ
   * ӥ󶡤ߤ뤳ȤǤ뤿ᡢӥ¦Ǥϡˤ
   * ӥѤǤȤϸ¤ʤΤȤƥӥڥ졼
   * ƤӽФɬפ롣
   *
   * SDO Service Consumer SDO/RTCʳSDO/RTCޤϥץ
   * 󤬥ӥμΤSDO/RTC˥֥ȻȤ
   * ޤץեͿ뤳ȤǡSDO/RTC¦饵ӥڥ졼
   * 󤬸ƤФ쳰SDO/RTCޤϥץꥱ󶡤뵡ǽ
   * Ǥ롣ޤ֥Ūʥ֥ȤͿ뤳ȤǡSDO/RTC¦
   * ΥХå¸뤿ˤѤ뤳ȤǤ롣󥷥塼
   * ޤϡץХȤϰۤʤꡢSDO Configuration󥿡ե
   * ɲáԤ롣Ϣ륪ڥ졼ϰʲΤȤǤ롣
   *
   * - Configuration::add_service_profile (in ServiceProfile sProfile)
   * - Configuration::remove_service_profile (in UniqueIdentifier id)
   *
   * SDO/RTCޤϥץꥱϡȤSDO Servcie
   * Provider λȤIDinterface typeץѥƥȤȤ
   * ServcieProfile ˥åȤǡadd_service_profile() ΰ
   * Ϳ뤳ȤǡSDO/RTC˥ӥͿ롣κݡIDUUID
   * ʤɰդIDǤʤФʤʤޤݤˤIDˤоݤ
   * ServiceProfileõ뤿ᡢӥ¦ǤϺޤID
   * ݻƤʤФʤʤ
   *
   * 
   *
   *
   *
   * @since 1.1.0
   *
   * @else
   *
   * @class SDO service administration class
   * @brief SDO service administration class
   *
   *
   * @since 1.1.0
   *
   * @endif
   */
  class SdoServiceAdmin
  {
  public:
    /*!
     * @if jp
     *
     * @brief 󥹥ȥ饯
     * 
     * 󥹥ȥ饯
     *
     * @param 
     * 
     * @else
     *
     * @brief Constructor
     * 
     * Constructor
     *
     * @param 
     *
     * @endif
     */
    SdoServiceAdmin(::RTC::RTObject_impl& rtobj);

    /*!
     * @if jp
     *
     * @brief ۥǥȥ饯
     * 
     * ۥǥȥ饯
     * 
     * @else
     *
     * @brief Virtual destractor
     *
     * Virtual destractor.
     *
     * @endif
     */
    virtual ~SdoServiceAdmin();
    
    /*!
     * @if jp
     *
     * @brief Service Consumer Factory Ͽ
     * 
     * @else
     *
     * @brief Add Service Consumer Factory
     *
     * @endif
     */
    bool addSdoServiceConsumerFactory();

    /*!
     * @if jp
     *
     * @brief Service Consumer Factory 
     * 
     * @else
     *
     * @brief Remove Service Consumer Factory
     *
     * @endif
     */
    bool removeSdoServiceConsumerFactory();
    
    /*!
     * @if jp
     *
     * @brief Service Consumer ɲä
     * 
     * @else
     *
     * @brief Add Service Consumer
     *
     * @endif
     */
    bool addSdoServiceConsumer(const SDOPackage::ServiceProfile& sProfile);
    
    /*!
     * @if jp
     *
     * @brief Service Consumer 
     * 
     * @else
     *
     * @brief Remove Service Consumer
     *
     * @endif
     */
    bool removeSdoServiceConsumer(const char* id);
    
protected:
    /*!
     * @if jp
     *
     * @brief Ĥ줿ӥɤĴ٤
     * 
     * @else
     *
     * @brief If it is allowed service type
     *
     * @endif
     */
    bool isAllowedConsumerType(const SDOPackage::ServiceProfile& sProfile);

    /*!
     * @if jp
     *
     * @brief ¸ߤ륵ӥɤĴ٤
     * 
     * @else
     *
     * @brief If it is existing service type
     *
     * @endif
     */
    bool isExistingConsumerType(const SDOPackage::ServiceProfile& sProfile);

    const std::string getUUID() const;
    

  private:
    RTC::RTObject_impl& m_rtobj;
    coil::vstring m_consumerTypes;
    bool m_allConsumerAllowed;
    
    /*!
     * @if jp
     * @brief Lock դ SDO ServiceProfileList
     * @else
     * @brief SDO ServiceProfileList with mutex lock
     * @endif
     */
    SDOPackage::ServiceProfileList m_providerProfiles;
    coil::Mutex m_provider_mutex;
    
    /*!
     * @if jp
     * @brief Lock դ SDO ServiceProfileList
     * @else
     * @brief SDO ServiceProfileList with mutex lock
     * @endif
     */
    std::vector<SdoServiceConsumerBase*> m_consumers;
    coil::Mutex m_consumer_mutex;

    /*!
     * @if jp
     * @brief logger
     * @else
     * @brief logger
     * @endif
     */
    ::RTC::Logger rtclog;
  };


};

#endif // RTC_SDOSERVICEADMIN_H
